<?php if (!defined('PmWiki')) exit();

/*

 PmWiki Voodoopad XML-RPC Interface

 -- Copyright --

 Copyright 2004, 2005 by James McDuffie (ttq3sxl02@sneakemail.com)

 -- License --

 GNU GPL

 -- Description --

 Provides an XML-RPC interface to PmWiki from a vpwiki api compliant client.
   
 -- History --

 Version 0.1 
 * Initial version

 Version 0.11
 * Made some changes to how keys are processed in GetPage to make finding
 a page easier when the user supplies keys with space characters in them

*/

define(PMWIKI_XMLRPC_VOODOOPAD, '0.11'); 

$VoodooPadFuncMap = 
  array 
  (
   "vpwiki.authenticate" => 
   array( "function" => array("VoodooPadXMLRPC", "Authenticate")),

   "vpwiki.getAllKeys" => 
   array( "function" => array("VoodooPadXMLRPC", "GetAllPageNames")),

   "vpwiki.getPage" => 
   array( "function" => array("VoodooPadXMLRPC", "GetPage")),

   "vpwiki.setPage" => 
   array( "function" => array("VoodooPadXMLRPC", "SetPage")),

   "vpwiki.putAttachment" => 
   array( "function" => array("VoodooPadXMLRPC", "PutAttachment")),
   );

// A class full of static methods is used to reduce namespace pollution
// Different XMLRPC interfaces might have similar method names
class VoodooPadXMLRPC {
  /******************************************************************
   VoodooPad Interface Functions
  ******************************************************************/

  // Authentificates the user by returning a true boolean if login is succesful
  function Authenticate($params) {
    $loginStructp=$params->getParam(0); $loginStruct=$loginStructp->scalarval();

    if(!VoodooPadXMLRPC::ValidateUserFromStruct($loginStruct)) {
      return new xmlrpcresp(new xmlrpcval(0, "boolean"));
    } else {
      return new xmlrpcresp(new xmlrpcval(1, "boolean"));
    }
  }

  // Retrieves all pagenames on the Wiki that the user has access to
  function GetAllPageNames($params) {
    $loginStructp=$params->getParam(0); $loginStruct=$loginStructp->scalarval();

    if(!VoodooPadXMLRPC::ValidateUserFromStruct($loginStruct)) {
      return new xmlrpcresp(0, 401, "Invalid username or password");
    }

    $pagelist = ListPages();
    $nameArray = Array();
    foreach($pagelist as $pagename) {
      // Make sure an AuthFunction has been defined before trying to
      // use it
      if(isset($AuthFunction)) {
	$accessible = $AuthFunction($pagename, 'read', false);
      } else {
	$accessible = true;
      }
   
      if($accessible) {
	$nameArray[] = new xmlrpcval($pagename);
      }
    }

    return new xmlrpcresp(new xmlrpcval($nameArray, "array"));
  }

  // Gets a specific page based upon a Voodoopad key. The key appears to
  // just be a lowercase version of a pagename. This method looks for the
  // first page that contains the key in its name
  function GetPage($params) {
    global $DefaultPage;

    $loginStructp=$params->getParam(0); $loginStruct=$loginStructp->scalarval();
    $keyp=$params->getParam(1); $key=$keyp->scalarval();

    if(!VoodooPadXMLRPC::ValidateUserFromStruct($loginStruct)) {
      return new xmlrpcresp(0, 401, "Invalid username or password");
    }

    // If the key is index then Voodoopad is looking for the default
    // page from the Wiki
    if($key == "index") {
      $key = strtolower($DefaultPage);
    }

    // Convert any slashes to dots
    $key = preg_replace("/\//", ".", $key);

    // Remove any spaces
    $key = preg_replace("/\s+/", "", $key);

    $pagelist = ListPages();
    foreach($pagelist as $currpage) {
      // If the key matches the current page then retrieve the page
      // set permissions and break
      if(preg_match("/$key/", strtolower($currpage))) {
	$pagename = $currpage;
	$page = RetrieveAuthPage($pagename, "read", false);
	
	// Can not return a page we do not have permission to
	if(!$page) return new xmlrpcresp(0, 401, "Permission denied");

	break;
      }
    }

    if(!isset($pagename)) {
      $page['name'] = $key;
    }

    $myStruct = VoodooPadXMLRPC::PageAsXMLStruct($page);
  
    return new xmlrpcresp($myStruct);
  }

  // Not implemented because there is no way to determine page name
  // based upon how Voodoopad uses lowercased keys
  //
  // It could be implemented but the resluting pages would not be CamelCase 
  // names
  function SetPage($params) {
    $loginStructp=$params->getParam(0); $loginStruct=$loginStructp->scalarval();
    $pageStructp=$params->getParam(1); $pageStruct=$pageStructp->scalarval();

    return new xmlrpcresp(0, 401, "Not implemented");
  }

  // Not implemented because PmWiki 
  function PutAttachment($params) {
    $loginStructp=$params->getParam(0); $loginStruct=$loginStructp->scalarval();
    $keyp=$params->getParam(1); $key=$keyp->scalarval();
    $fileStructp=$params->getParam(2); $fileStruct=$fileStructp->scalarval();

    return new xmlrpcresp(0, 401, "Not implemented");    
  }

  /******************************************************************
   VoodooPad Helper Functions
  ******************************************************************/

  // Simply pulls the username and password from the login struct then
  // calls the validation function  
  function ValidateUserFromStruct($loginStruct) {
    $username = $loginStruct['name']->scalarval();
    $password = $loginStruct['password']->scalarval();

    return ValidateUser($username, $password);
  }

  // Helper function that returns a PmWiki page as a XML structure
  function PageAsXMLStruct($page) {

    // Change Unix time into ISO8601 format
    $lastmodified = strftime("%Y-%m-%dT%H:%M:%S%z",$page['time']);

    return 
      new xmlrpcval(
		    array("key" => new xmlrpcval($page['name']),
			  "page" => new xmlrpcval($page['text']),
			  "lastModified" => new xmlrpcval($lastmodified, "dateTime.iso8601"),
			  "version" => new xmlrpcval($page['version']),
			  ),"struct");
  }


} // End of VoodooPadXMLRPC Class

?>
